@@ -475,39 +475,6 @@ def start():
         f.write("\n    0x3072cfa1: \"pyrogram.api.core.GzipPacked\",")
         f.write("\n    0x5bb8e511: \"pyrogram.api.core.Message\",")
 
-        f.write("\n    0xb0700000: \"pyrogram.client.types.Update\",")
-        f.write("\n    0xb0700001: \"pyrogram.client.types.User\",")
-        f.write("\n    0xb0700002: \"pyrogram.client.types.Chat\",")
-        f.write("\n    0xb0700003: \"pyrogram.client.types.Message\",")
-        f.write("\n    0xb0700004: \"pyrogram.client.types.MessageEntity\",")
-        f.write("\n    0xb0700005: \"pyrogram.client.types.PhotoSize\",")
-        f.write("\n    0xb0700006: \"pyrogram.client.types.Audio\",")
-        f.write("\n    0xb0700007: \"pyrogram.client.types.Document\",")
-        f.write("\n    0xb0700008: \"pyrogram.client.types.Video\",")
-        f.write("\n    0xb0700009: \"pyrogram.client.types.Voice\",")
-        f.write("\n    0xb0700010: \"pyrogram.client.types.VideoNote\",")
-        f.write("\n    0xb0700011: \"pyrogram.client.types.Contact\",")
-        f.write("\n    0xb0700012: \"pyrogram.client.types.Location\",")
-        f.write("\n    0xb0700013: \"pyrogram.client.types.Venue\",")
-        f.write("\n    0xb0700014: \"pyrogram.client.types.UserProfilePhotos\",")
-        f.write("\n    0xb0700015: \"pyrogram.client.types.ChatPhoto\",")
-        f.write("\n    0xb0700016: \"pyrogram.client.types.ChatMember\",")
-        f.write("\n    0xb0700017: \"pyrogram.client.types.Sticker\",")
-        f.write("\n    0xb0700018: \"pyrogram.client.types.bots.ForceReply\",")
-        f.write("\n    0xb0700019: \"pyrogram.client.types.bots.InlineKeyboardButton\",")
-        f.write("\n    0xb0700020: \"pyrogram.client.types.bots.InlineKeyboardMarkup\",")
-        f.write("\n    0xb0700021: \"pyrogram.client.types.bots.KeyboardButton\",")
-        f.write("\n    0xb0700022: \"pyrogram.client.types.bots.ReplyKeyboardMarkup\",")
-        f.write("\n    0xb0700023: \"pyrogram.client.types.bots.ReplyKeyboardRemove\",")
-        f.write("\n    0xb0700024: \"pyrogram.client.types.CallbackQuery\",")
-        f.write("\n    0xb0700025: \"pyrogram.client.types.Animation\",")
-        f.write("\n    0xb0700026: \"pyrogram.client.types.Messages\",")
-        f.write("\n    0xb0700027: \"pyrogram.client.types.Photo\",")
-        f.write("\n    0xb0700028: \"pyrogram.client.types.Dialog\",")
-        f.write("\n    0xb0700029: \"pyrogram.client.types.Dialogs\",")
-        f.write("\n    0xb0700030: \"pyrogram.client.types.ChatMembers\",")
-        f.write("\n    0xb0700031: \"pyrogram.client.types.UserStatus\"")
-
         f.write("\n}\n")
 
     for k, v in namespaces.items():
