@@ -18,7 +18,6 @@
 
 import base64
 import binascii
-import getpass
 import json
 import logging
 import math
@@ -37,6 +36,7 @@ from importlib import import_module
 from pathlib import Path
 from signal import signal, SIGINT, SIGTERM, SIGABRT
 from threading import Thread
+from typing import Union, List
 
 from pyrogram.api import functions, types
 from pyrogram.api.core import Object
@@ -45,9 +45,12 @@ from pyrogram.api.errors import (
     PhoneNumberUnoccupied, PhoneCodeInvalid, PhoneCodeHashEmpty,
     PhoneCodeExpired, PhoneCodeEmpty, SessionPasswordNeeded,
     PasswordHashInvalid, FloodWait, PeerIdInvalid, FirstnameInvalid, PhoneNumberBanned,
-    VolumeLocNotFound, UserMigrate, FileIdInvalid, ChannelPrivate)
+    VolumeLocNotFound, UserMigrate, FileIdInvalid, ChannelPrivate, PhoneNumberOccupied,
+    PasswordRecoveryNa, PasswordEmpty
+)
 from pyrogram.client.handlers import DisconnectHandler
 from pyrogram.client.handlers.handler import Handler
+from pyrogram.client.methods.password.utils import compute_check
 from pyrogram.crypto import AES
 from pyrogram.session import Auth, Session
 from .dispatcher import Dispatcher
@@ -152,7 +155,7 @@ class Client(Methods, BaseClient):
 
     def __init__(self,
                  session_name: str,
-                 api_id: int or str = None,
+                 api_id: Union[int, str] = None,
                  api_hash: str = None,
                  app_version: str = None,
                  device_model: str = None,
@@ -162,7 +165,7 @@ class Client(Methods, BaseClient):
                  proxy: dict = None,
                  test_mode: bool = False,
                  phone_number: str = None,
-                 phone_code: str or callable = None,
+                 phone_code: Union[str, callable] = None,
                  password: str = None,
                  force_sms: bool = False,
                  first_name: str = None,
@@ -198,10 +201,9 @@ class Client(Methods, BaseClient):
         self.dispatcher = Dispatcher(self, workers)
 
     def __enter__(self):
-        self.start()
-        return self
+        return self.start()
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, *args):
         self.stop()
 
     @property
@@ -294,6 +296,8 @@ class Client(Methods, BaseClient):
         mimetypes.init()
         Syncer.add(self)
 
+        return self
+
     def stop(self):
         """Use this method to manually stop the Client.
         Requires no parameters.
@@ -331,6 +335,8 @@ class Client(Methods, BaseClient):
         self.is_started = False
         self.session.stop()
 
+        return self
+
     def idle(self, stop_signals: tuple = (SIGINT, SIGTERM, SIGABRT)):
         """Blocks the program execution until one of the signals are received,
         then gently stop the Client by closing the underlying connection.
@@ -364,7 +370,7 @@ class Client(Methods, BaseClient):
         self.start()
         self.idle()
 
-    def add_handler(self, handler, group: int = 0):
+    def add_handler(self, handler: Handler, group: int = 0):
         """Use this method to register an update handler.
 
         You can register multiple handlers, but at most one handler within a group
@@ -388,7 +394,7 @@ class Client(Methods, BaseClient):
 
         return handler, group
 
-    def remove_handler(self, handler, group: int = 0):
+    def remove_handler(self, handler: Handler, group: int = 0):
         """Removes a previously-added update handler.
 
         Make sure to provide the right group that the handler was added in. You can use
@@ -434,6 +440,8 @@ class Client(Methods, BaseClient):
         else:
             self.user_id = r.user.id
 
+            print("Logged in successfully as @{}".format(r.user.username))
+
     def authorize_user(self):
         phone_number_invalid_raises = self.phone_number is not None
         phone_code_invalid_raises = self.phone_code is not None
@@ -524,16 +532,8 @@ class Client(Methods, BaseClient):
 
             try:
                 if phone_registered:
-                    r = self.send(
-                        functions.auth.SignIn(
-                            self.phone_number,
-                            phone_code_hash,
-                            self.phone_code
-                        )
-                    )
-                else:
                     try:
-                        self.send(
+                        r = self.send(
                             functions.auth.SignIn(
                                 self.phone_number,
                                 phone_code_hash,
@@ -541,20 +541,27 @@ class Client(Methods, BaseClient):
                             )
                         )
                     except PhoneNumberUnoccupied:
-                        pass
-
+                        log.warning("Phone number unregistered")
+                        phone_registered = False
+                        continue
+                else:
                     self.first_name = self.first_name if self.first_name is not None else input("First name: ")
                     self.last_name = self.last_name if self.last_name is not None else input("Last name: ")
 
-                    r = self.send(
-                        functions.auth.SignUp(
-                            self.phone_number,
-                            phone_code_hash,
-                            self.phone_code,
-                            self.first_name,
-                            self.last_name
+                    try:
+                        r = self.send(
+                            functions.auth.SignUp(
+                                self.phone_number,
+                                phone_code_hash,
+                                self.phone_code,
+                                self.first_name,
+                                self.last_name
+                            )
                         )
-                    )
+                    except PhoneNumberOccupied:
+                        log.warning("Phone number already registered")
+                        phone_registered = True
+                        continue
             except (PhoneCodeInvalid, PhoneCodeEmpty, PhoneCodeExpired, PhoneCodeHashEmpty) as e:
                 if phone_code_invalid_raises:
                     raise
@@ -569,21 +576,46 @@ class Client(Methods, BaseClient):
                     self.first_name = None
             except SessionPasswordNeeded as e:
                 print(e.MESSAGE)
-                r = self.send(functions.account.GetPassword())
 
                 while True:
                     try:
+                        r = self.send(functions.account.GetPassword())
 
                         if self.password is None:
                             print("Hint: {}".format(r.hint))
-                            self.password = getpass.getpass("Enter password: ")
 
-                        if type(self.password) is str:
-                            self.password = r.current_salt + self.password.encode() + r.current_salt
+                            self.password = input("Enter password (empty to recover): ")
 
-                        password_hash = sha256(self.password).digest()
+                            if self.password == "":
+                                r = self.send(functions.auth.RequestPasswordRecovery())
 
-                        r = self.send(functions.auth.CheckPassword(password_hash))
+                                print("An e-mail containing the recovery code has been sent to {}".format(
+                                    r.email_pattern
+                                ))
+
+                                r = self.send(
+                                    functions.auth.RecoverPassword(
+                                        code=input("Enter password recovery code: ")
+                                    )
+                                )
+                            else:
+                                r = self.send(
+                                    functions.auth.CheckPassword(
+                                        password=compute_check(r, self.password)
+                                    )
+                                )
+                    except PasswordEmpty as e:
+                        if password_hash_invalid_raises:
+                            raise
+                        else:
+                            print(e.MESSAGE)
+                            self.password = None
+                    except PasswordRecoveryNa as e:
+                        if password_hash_invalid_raises:
+                            raise
+                        else:
+                            print(e.MESSAGE)
+                            self.password = None
                     except PasswordHashInvalid as e:
                         if password_hash_invalid_raises:
                             raise
@@ -596,6 +628,7 @@ class Client(Methods, BaseClient):
                         else:
                             print(e.MESSAGE.format(x=e.x))
                             time.sleep(e.x)
+                            self.password = None
                     except Exception as e:
                         log.error(e, exc_info=True)
                     else:
@@ -618,9 +651,11 @@ class Client(Methods, BaseClient):
         self.password = None
         self.user_id = r.user.id
 
-        print("Login successful")
+        print("Logged in successfully as {}".format(r.user.first_name))
 
-    def fetch_peers(self, entities: list):
+    def fetch_peers(self, entities: List[Union[types.User,
+                                               types.Chat, types.ChatForbidden,
+                                               types.Channel, types.ChannelForbidden]]):
         for entity in entities:
             if isinstance(entity, types.User):
                 user_id = entity.id
@@ -882,7 +917,10 @@ class Client(Methods, BaseClient):
 
         log.debug("{} stopped".format(name))
 
-    def send(self, data: Object, retries: int = Session.MAX_RETRIES, timeout: float = Session.WAIT_TIMEOUT):
+    def send(self,
+             data: Object,
+             retries: int = Session.MAX_RETRIES,
+             timeout: float = Session.WAIT_TIMEOUT):
         """Use this method to send Raw Function queries.
 
         This method makes possible to manually call every single Telegram API method in a low-level manner.
@@ -1041,7 +1079,8 @@ class Client(Methods, BaseClient):
                 indent=4
             )
 
-    def get_initial_dialogs_chunk(self, offset_date: int = 0):
+    def get_initial_dialogs_chunk(self,
+                                  offset_date: int = 0):
         while True:
             try:
                 r = self.send(
@@ -1073,12 +1112,13 @@ class Client(Methods, BaseClient):
 
         self.get_initial_dialogs_chunk()
 
-    def resolve_peer(self, peer_id: int or str):
+    def resolve_peer(self,
+                     peer_id: Union[int, str]):
         """Use this method to get the InputPeer of a known peer_id.
 
-        This is a utility method intended to be used only when working with Raw Functions (i.e: a Telegram API method
-        you wish to use which is not available yet in the Client class as an easy-to-use method), whenever an InputPeer
-        type is required.
+        This is a utility method intended to be used **only** when working with Raw Functions (i.e: a Telegram API
+        method you wish to use which is not available yet in the Client class as an easy-to-use method), whenever an
+        InputPeer type is required.
 
         Args:
             peer_id (``int`` | ``str``):
@@ -1092,35 +1132,58 @@ class Client(Methods, BaseClient):
             :class:`Error <pyrogram.Error>` in case of a Telegram RPC error.
             ``KeyError`` in case the peer doesn't exist in the internal database.
         """
-        if type(peer_id) is str:
-            if peer_id in ("self", "me"):
-                return types.InputPeerSelf()
-
-            peer_id = re.sub(r"[@+\s]", "", peer_id.lower())
-
-            try:
-                int(peer_id)
-            except ValueError:
-                if peer_id not in self.peers_by_username:
-                    self.send(functions.contacts.ResolveUsername(peer_id))
-
-                return self.peers_by_username[peer_id]
-            else:
-                try:
-                    return self.peers_by_phone[peer_id]
-                except KeyError:
-                    raise PeerIdInvalid
-
-        try:  # User
+        try:
             return self.peers_by_id[peer_id]
         except KeyError:
-            try:  # Chat
-                return self.peers_by_id[-peer_id]
+            if type(peer_id) is str:
+                if peer_id in ("self", "me"):
+                    return types.InputPeerSelf()
+
+                peer_id = re.sub(r"[@+\s]", "", peer_id.lower())
+
+                try:
+                    int(peer_id)
+                except ValueError:
+                    if peer_id not in self.peers_by_username:
+                        self.send(
+                            functions.contacts.ResolveUsername(
+                                username=peer_id
+                            )
+                        )
+
+                    return self.peers_by_username[peer_id]
+                else:
+                    try:
+                        return self.peers_by_phone[peer_id]
+                    except KeyError:
+                        raise PeerIdInvalid
+
+            if peer_id > 0:
+                self.fetch_peers(
+                    self.send(
+                        functions.users.GetUsers(
+                            id=[types.InputUser(peer_id, 0)]
+                        )
+                    )
+                )
+            else:
+                if str(peer_id).startswith("-100"):
+                    self.send(
+                        functions.channels.GetChannels(
+                            id=[types.InputChannel(int(str(peer_id)[4:]), 0)]
+                        )
+                    )
+                else:
+                    self.send(
+                        functions.messages.GetChats(
+                            id=[-peer_id]
+                        )
+                    )
+
+            try:
+                return self.peers_by_id[peer_id]
             except KeyError:
-                try:  # Channel
-                    return self.peers_by_id[int("-100" + str(peer_id))]
-                except (KeyError, ValueError):
-                    raise PeerIdInvalid
+                raise PeerIdInvalid
 
     def save_file(self,
                   path: str,
@@ -1128,15 +1191,60 @@ class Client(Methods, BaseClient):
                   file_part: int = 0,
                   progress: callable = None,
                   progress_args: tuple = ()):
+        """Use this method to upload a file onto Telegram servers, without actually sending the message to anyone.
+
+        This is a utility method intended to be used **only** when working with Raw Functions (i.e: a Telegram API
+        method you wish to use which is not available yet in the Client class as an easy-to-use method), whenever an
+        InputFile type is required.
+
+        Args:
+            path (``str``):
+                The path of the file you want to upload that exists on your local machine.
+
+            file_id (``int``, *optional*):
+                In case a file part expired, pass the file_id and the file_part to retry uploading that specific chunk.
+
+            file_part (``int``, *optional*):
+                In case a file part expired, pass the file_id and the file_part to retry uploading that specific chunk.
+
+            progress (``callable``, *optional*):
+                Pass a callback function to view the upload progress.
+                The function must take *(client, current, total, \*args)* as positional arguments (look at the section
+                below for a detailed description).
+
+            progress_args (``tuple``, *optional*):
+                Extra custom arguments for the progress callback function. Useful, for example, if you want to pass
+                a chat_id and a message_id in order to edit a message with the updated progress.
+
+        Other Parameters:
+            client (:obj:`Client <pyrogram.Client>`):
+                The Client itself, useful when you want to call other API methods inside the callback function.
+
+            current (``int``):
+                The amount of bytes uploaded so far.
+
+            total (``int``):
+                The size of the file.
+
+            *args (``tuple``, *optional*):
+                Extra custom arguments as defined in the *progress_args* parameter.
+                You can either keep *\*args* or add every single extra argument in your function signature.
+
+        Returns:
+            On success, the uploaded file is returned in form of an InputFile object.
+
+        Raises:
+            :class:`Error <pyrogram.Error>` in case of a Telegram RPC error.
+        """
         part_size = 512 * 1024
         file_size = os.path.getsize(path)
-        
+
         if file_size == 0:
             raise ValueError("File size equals to 0 B")
-        
+
         if file_size > 1500 * 1024 * 1024:
             raise ValueError("Telegram doesn't support uploading files bigger than 1500 MiB")
-            
+
         file_total_parts = int(math.ceil(file_size / part_size))
         is_big = True if file_size > 10 * 1024 * 1024 else False
         is_missing_part = True if file_id is not None else False
@@ -1211,10 +1319,9 @@ class Client(Methods, BaseClient):
                  volume_id: int = None,
                  local_id: int = None,
                  secret: int = None,
-                 version: int = 0,
                  size: int = None,
                  progress: callable = None,
-                 progress_args: tuple = None) -> str:
+                 progress_args: tuple = ()) -> str:
         with self.media_sessions_lock:
             session = self.media_sessions.get(dc_id, None)
 
@@ -1259,13 +1366,14 @@ class Client(Methods, BaseClient):
             location = types.InputFileLocation(
                 volume_id=volume_id,
                 local_id=local_id,
-                secret=secret
+                secret=secret,
+                file_reference=b""
             )
         else:  # Any other file can be more easily accessed by id and access_hash
             location = types.InputDocumentFileLocation(
                 id=id,
                 access_hash=access_hash,
-                version=version
+                file_reference=b""
             )
 
         limit = 1024 * 1024
@@ -1296,7 +1404,7 @@ class Client(Methods, BaseClient):
                         offset += limit
 
                         if progress:
-                            progress(self, min(offset, size), size, *progress_args)
+                            progress(self, min(offset, size) if size != 0 else offset, size, *progress_args)
 
                         r = session.send(
                             functions.upload.GetFile(
@@ -1378,7 +1486,7 @@ class Client(Methods, BaseClient):
                             offset += limit
 
                             if progress:
-                                progress(self, min(offset, size), size, *progress_args)
+                                progress(self, min(offset, size) if size != 0 else offset, size, *progress_args)
 
                             if len(chunk) < limit:
                                 break
