@@ -16,16 +16,18 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.
 
+from typing import Union
+
+import pyrogram
 from pyrogram.api import functions, types
-from pyrogram.client.ext import utils
 from ...ext import BaseClient
 
 
 class KickChatMember(BaseClient):
     def kick_chat_member(self,
-                         chat_id: int or str,
-                         user_id: int or str,
-                         until_date: int = 0):
+                         chat_id: Union[int, str],
+                         user_id: Union[int, str],
+                         until_date: int = 0) -> "pyrogram.Message":
         """Use this method to kick a user from a group, a supergroup or a channel.
         In the case of supergroups and channels, the user will not be able to return to the group on their own using
         invite links, etc., unless unbanned first. You must be an administrator in the chat for this to work and must
@@ -86,7 +88,7 @@ class KickChatMember(BaseClient):
 
         for i in r.updates:
             if isinstance(i, (types.UpdateNewMessage, types.UpdateNewChannelMessage)):
-                return utils.parse_messages(
+                return pyrogram.Message._parse(
                     self, i.message,
                     {i.id: i for i in r.users},
                     {i.id: i for i in r.chats}
