@@ -20,7 +20,9 @@ import binascii
 import mimetypes
 import os
 import struct
+from typing import Union
 
+import pyrogram
 from pyrogram.api import functions, types
 from pyrogram.api.errors import FileIdInvalid
 from pyrogram.client.ext import BaseClient, utils
@@ -28,14 +30,15 @@ from pyrogram.client.types import (
     InputMediaPhoto, InputMediaVideo, InputMediaAudio,
     InputMediaAnimation, InputMediaDocument
 )
+from pyrogram.client.types.input_media import InputMedia
 
 
 class EditMessageMedia(BaseClient):
     def edit_message_media(self,
-                           chat_id: int or str,
+                           chat_id: Union[int, str],
                            message_id: int,
-                           media,
-                           reply_markup=None):
+                           media: InputMedia,
+                           reply_markup: "pyrogram.InlineKeyboardMarkup" = None) -> "pyrogram.Message":
         """Use this method to edit audio, document, photo, or video messages.
 
         If a message is a part of a message album, then it can be edited only to a photo or a video. Otherwise,
@@ -81,7 +84,8 @@ class EditMessageMedia(BaseClient):
                 media = types.InputMediaPhoto(
                     id=types.InputPhoto(
                         id=media.photo.id,
-                        access_hash=media.photo.access_hash
+                        access_hash=media.photo.access_hash,
+                        file_reference=b""
                     )
                 )
             elif media.media.startswith("http"):
@@ -107,7 +111,8 @@ class EditMessageMedia(BaseClient):
                     media = types.InputMediaPhoto(
                         id=types.InputPhoto(
                             id=unpacked[2],
-                            access_hash=unpacked[3]
+                            access_hash=unpacked[3],
+                            file_reference=b""
                         )
                     )
 
@@ -135,7 +140,8 @@ class EditMessageMedia(BaseClient):
                 media = types.InputMediaDocument(
                     id=types.InputDocument(
                         id=media.document.id,
-                        access_hash=media.document.access_hash
+                        access_hash=media.document.access_hash,
+                        file_reference=b""
                     )
                 )
             elif media.media.startswith("http"):
@@ -161,7 +167,8 @@ class EditMessageMedia(BaseClient):
                     media = types.InputMediaDocument(
                         id=types.InputDocument(
                             id=unpacked[2],
-                            access_hash=unpacked[3]
+                            access_hash=unpacked[3],
+                            file_reference=b""
                         )
                     )
 
@@ -188,7 +195,8 @@ class EditMessageMedia(BaseClient):
                 media = types.InputMediaDocument(
                     id=types.InputDocument(
                         id=media.document.id,
-                        access_hash=media.document.access_hash
+                        access_hash=media.document.access_hash,
+                        file_reference=b""
                     )
                 )
             elif media.media.startswith("http"):
@@ -214,7 +222,8 @@ class EditMessageMedia(BaseClient):
                     media = types.InputMediaDocument(
                         id=types.InputDocument(
                             id=unpacked[2],
-                            access_hash=unpacked[3]
+                            access_hash=unpacked[3],
+                            file_reference=b""
                         )
                     )
 
@@ -243,7 +252,8 @@ class EditMessageMedia(BaseClient):
                 media = types.InputMediaDocument(
                     id=types.InputDocument(
                         id=media.document.id,
-                        access_hash=media.document.access_hash
+                        access_hash=media.document.access_hash,
+                        file_reference=b""
                     )
                 )
             elif media.media.startswith("http"):
@@ -269,7 +279,8 @@ class EditMessageMedia(BaseClient):
                     media = types.InputMediaDocument(
                         id=types.InputDocument(
                             id=unpacked[2],
-                            access_hash=unpacked[3]
+                            access_hash=unpacked[3],
+                            file_reference=b""
                         )
                     )
 
@@ -291,7 +302,8 @@ class EditMessageMedia(BaseClient):
                 media = types.InputMediaDocument(
                     id=types.InputDocument(
                         id=media.document.id,
-                        access_hash=media.document.access_hash
+                        access_hash=media.document.access_hash,
+                        file_reference=b""
                     )
                 )
             elif media.media.startswith("http"):
@@ -317,7 +329,8 @@ class EditMessageMedia(BaseClient):
                     media = types.InputMediaDocument(
                         id=types.InputDocument(
                             id=unpacked[2],
-                            access_hash=unpacked[3]
+                            access_hash=unpacked[3],
+                            file_reference=b""
                         )
                     )
 
@@ -333,7 +346,7 @@ class EditMessageMedia(BaseClient):
 
         for i in r.updates:
             if isinstance(i, (types.UpdateEditMessage, types.UpdateEditChannelMessage)):
-                return utils.parse_messages(
+                return pyrogram.Message._parse(
                     self, i.message,
                     {i.id: i for i in r.users},
                     {i.id: i for i in r.chats}
