@@ -16,8 +16,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.
 
-from .ctr import CTR
-from .ige import IGE
+from .aes import AES
 from .kdf import KDF
 from .prime import Prime
 from .rsa import RSA
